@@ -19,6 +19,8 @@ from flask import (Flask, render_template, send_from_directory,
                    request, jsonify, Response)
 from werkzeug.serving import WSGIRequestHandler
 
+import util
+
 
 def determine_path():
     """Borrowed from wxglade.py"""
@@ -33,6 +35,7 @@ def determine_path():
         sys.exit()
 
 # This is essential when run from distutils and does no harm otherwise.
+ORIGINAL_DIR = os.getcwd()
 path = determine_path()
 os.chdir(path)
 
@@ -135,7 +138,12 @@ def get_contents(side):
 
     try:
         abs_path = os.path.join(A_DIR if side == 'a' else B_DIR, path)
-        contents = open(abs_path).read()
+        is_binary = util.is_binary_file(abs_path)
+        if is_binary:
+          size = os.path.getsize(abs_path)
+          contents = "Binary file (%d bytes)" % size
+        else:
+          contents = open(abs_path).read()
         return Response(contents, mimetype='text/plain')
     except Exception:
         e = {"code": "read-error",
@@ -205,11 +213,20 @@ def pick_a_port():
     return port
 
 
+def adjust_path(path):
+    '''Changes relative paths to be abs w/r/t/ the original cwd.'''
+    global ORIGINAL_DIR
+    if os.path.isabs(path):
+        return path
+    else:
+        return os.path.join(ORIGINAL_DIR, path)
+
+
 def run():
     global A_DIR, B_DIR, DIFF, PORT
     assert len(sys.argv) == 3
-    A_DIR = sys.argv[1]
-    B_DIR = sys.argv[2]
+    A_DIR = adjust_path(sys.argv[1])
+    B_DIR = adjust_path(sys.argv[2])
     if not os.path.isdir(A_DIR) or not os.path.isdir(B_DIR):
         usage_and_die()
 
